︠b97d3da9-9ffa-4255-8c12-000f82469f49︠
%cython
cimport cython

cdef extern from "math.h":
    double exp(double)

cdef double gascon = 8.3144
cdef double mw_h2o = 0.018015
cdef double gascon_h2o = gascon / mw_h2o
cdef double sigma_at_tk = 0.072
cdef double rho_h2o = 997.0
cdef double tk_aerosol = 298.15
cdef double kappa = 1.0
cdef double kelvin = 2.0 * sigma_at_tk / (gascon_h2o * rho_h2o * tk_aerosol)
                                            
@cython.cdivision(True)
cpdef double petters_solve_for_rw(double x, double rd, double rh):
    return rh - exp(kelvin/x) * (x**3 - rd**3) / (x**3 - rd**3 * (1.0 - kappa))
︡6e79417f-b67d-43b9-a501-d3c807545c4e︡︡
︠f0afd0da-4af2-45ed-b3ef-7118e662211d︠
from scipy import optimize
# These are input to the petters_solve_for_rw
rd = 5.75e-08
rh = 0.95
︡3b0e67eb-e66d-4c45-b9c5-033336254769︡︡
︠f0f563bc-cc85-4ef2-802a-889599f4c708︠
# First, find a solution
optimize.fsolve(petters_solve_for_rw, rd, args=(rd, rh))
︡1d3c86ed-f67a-4317-ae31-cc2656dfac3e︡{"stdout": "1.4972782377152967e-07"}︡
︠f85ac960-b1d1-4831-97cb-e15effde866a︠
timeit('optimize.fsolve(petters_solve_for_rw, rd, args=(rd, rh))')
︡ab42afa2-a7e5-447a-9cab-abbae539d24b︡{"stdout": "625 loops, best of 3: 133 \u00b5s per loop"}︡
︠23e8a455-d6bc-4c1a-9069-68e28ab384ad︠
# my local scipy v0.10.dev can't produce this result
optimize.newton(petters_solve_for_rw, rd, args=(rd, rh), tol=1.e-10)
︡92a399ba-aada-4e01-841d-7439a2e601dc︡{"stdout": "1.49726676560525e-7"}︡
︠d70cfd62-a7a6-4fe7-8151-6adc891a4f83︠
timeit('optimize.newton(petters_solve_for_rw, rd, args=(rd, rh))')
︡cae0a034-8cb1-494a-bf02-b2a99cb01068︡{"stdout": "625 loops, best of 3: 26.2 \u00b5s per loop"}︡
︠9b8d7f29-95a8-4fa7-91d4-bf1c187047d2︠
#SAGE scipy version
from scipy import __version__
print __version__
#newton function seems changed in between versions.
︡f1e9392a-112b-4d6d-921a-5902d9a466a2︡{"stdout": "0.7.0"}︡
︠9edf7394-70d0-487b-ad50-67f51425ed3c︠
# Slightly modified newton -- 1.e-4 changed to 1.e-10
# Newton --using the secant method
def pnewton(func, x0, args=(), tol=1.e-10, maxiter=50):
    """Find a zero using the secant method.

    Find a zero of the function `func` given a nearby starting point `x0`.

    Parameters
    ----------
    func : function
        The function whose zero is wanted. It must be a function of a
        single variable of the form f(x,a,b,c...), where a,b,c... are extra
        arguments that can be passed in the `args` parameter.
    x0 : float
        An initial estimate of the zero that should be somewhere near the
        actual zero.
    args : tuple, optional
        Extra arguments to be used in the function call.
    tol : float, optional
        The allowable error of the zero value.
    maxiter : int, optional
        Maximum number of iterations.

    Returns
    -------
    zero : float
        Estimated location where function is zero.
    """
    # Secant method
    p0 = x0
    if x0 >= 0:
        p1 = x0*(1 + 1e-10) + 1e-10
    else:
        p1 = x0*(1 + 1e-10) - 1e-10
    q0 = func(*((p0,) + args))
    q1 = func(*((p1,) + args))
    for iter in range(maxiter):
        if q1 == q0:
            if p1 != p0:
                return (p1 + p0)/2.0
        else:
            p = p1 - q1*(p1 - p0)/(q1 - q0)
        if abs(p - p1) < tol:
            return p
        p0 = p1
        q0 = q1
        p1 = p
        q1 = func(*((p1,) + args))
︡62070f7c-ffd3-4761-a5d4-31f4eecd5f64︡︡
︠05715881-96a2-4f5e-aac6-0634fa0597bf︠
# Now the result is correct.
pnewton(petters_solve_for_rw, rd, args=(rd, rh))
︡727815ed-3b62-4c5c-8efe-f3f2c9cb6c90︡{"stdout": "1.49726708556492e-7"}︡
︠04450e98-6de4-4921-9c09-e905efbc1885︠
timeit('pnewton(petters_solve_for_rw, rd, args=(rd, rh))')
︡05ca0449-8e89-415d-bc0c-a59e985f365f︡{"stdout": "625 loops, best of 3: 79.3 \u00b5s per loop"}︡
︠da9c2c26-adf3-4ec0-ae7a-de54f0e6de4a︠
%cython
# An effort to cythonize the pnewton()
cpdef double cnewton(func, double x0, args=(), double tol=1.e-10, int maxiter=50):
    """Find a zero using the secant method.

    Find a zero of the function `func` given a nearby starting point `x0`.

    Parameters
    ----------
    func : function
        The function whose zero is wanted. It must be a function of a
        single variable of the form f(x,a,b,c...), where a,b,c... are extra
        arguments that can be passed in the `args` parameter.
    x0 : float
        An initial estimate of the zero that should be somewhere near the
        actual zero.
    args : tuple, optional
        Extra arguments to be used in the function call.
    tol : float, optional
        The allowable error of the zero value.
    maxiter : int, optional
        Maximum number of iterations.

    Returns
    -------
    zero : float
        Estimated location where function is zero.
    """
    # Secant method
    p0 = x0
    if x0 >= 0:
        p1 = x0*(1 + 1e-10) + 1e-10
    else:
        p1 = x0*(1 + 1e-10) - 1e-10
    q0 = func(*((p0,) + args))
    q1 = func(*((p1,) + args))
    for iter in range(maxiter):
        if q1 == q0:
            if p1 != p0:
                return (p1 + p0)/2.0
        else:
            p = p1 - q1*(p1 - p0)/(q1 - q0)
        if abs(p - p1) < tol:
            return p
        p0 = p1
        q0 = q1
        p1 = p
        q1 = func(*((p1,) + args))
︡de034ce8-6790-49ad-8b2e-6a0ab1f56f10︡︡
︠3d15066b-b622-406a-abe0-d92d2df1c870︠
cnewton(petters_solve_for_rw, rd, args=(rd, rh))
︡8665a334-2e4e-4bbd-8a7b-e9d3500a889e︡{"stdout": "1.4972670855649154e-07"}︡
︠c3291ec2-8efc-44c4-a830-ddab158247d9︠
timeit('cnewton(petters_solve_for_rw, rd, args=(rd, rh))')
︡05f12697-45be-474e-8914-85e8dadc1b17︡{"stdout": "625 loops, best of 3: 9.05 \u00b5s per loop"}︡
